package org.ow2.cmi.agent;

import java.text.DateFormat;
import java.util.Date;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.Constants;
import org.osgi.framework.ServiceReference;
import org.osgi.service.log.LogEntry;
import org.osgi.service.log.LogListener;
import org.osgi.service.log.LogReaderService;
import org.osgi.service.log.LogService;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;


public class Activator implements BundleActivator {

     /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(Activator.class);

    /**
     * List of bundles to be started by the agent.
     * WARNING: Sorting matters !
     */
    private static final String[] BUNDLE_SYMBOLIC_NAMES = {
        "org.ow2.cmi.osgi.api.client",
        "org.ow2.cmi.osgi.api.server",
        "org.ow2.cmi.osgi.all",
        "org.ow2.cmi.osgi.extension.manager",
        "org.ow2.cmi.osgi.loader"
       };

    /**
     * Starts the Cmi bundle in the right order.
     * {@inheritDoc}
     * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
     */
    @SuppressWarnings("unchecked")
    public void start(final BundleContext bc) throws Exception {

         // Lookup logreaderservice and register a simpleloglistener
        ServiceReference logServiceReference = bc.getServiceReference(LogReaderService.class.getName());

        if (logServiceReference != null){
            LogReaderService logReaderService = (LogReaderService) bc.getService(logServiceReference);
            logReaderService.addLogListener(new SimpleLogListenerImpl());
        }

        Bundle[] bundles = bc.getBundles();

        // Start Bundles in the array order
        for (int i = 0; i < BUNDLE_SYMBOLIC_NAMES.length; i++) {

            // 1. Start the bundle with the right symbolic name
            Bundle bundle = null;
            for (int j = 0; j < bundles.length; j++) {
                bundle = bundles[j];
                if (BUNDLE_SYMBOLIC_NAMES[i].equals(bundle.getSymbolicName())) {
                    logger.debug("Starting Bundle", bundle);
                    // First start the bundle
                    if (bundle.getState() == Bundle.RESOLVED || bundle.getState() == Bundle.INSTALLED) {
                        bundle.start();
                    }
                }
            }
        }
    }

    /**
     * Stops the CMI bundle in the right order. {@inheritDoc}
     *
     * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
     */
    public void stop(final BundleContext bc) throws Exception {
        Bundle[] bundles = bc.getBundles();

        // Traverse the array starting by the end
        for (int i = BUNDLE_SYMBOLIC_NAMES.length - 1; i >= 0; i--) {
            Bundle bundle = null;
            for (int j = 0; j < bundles.length && bundle == null; j++) {
                if (BUNDLE_SYMBOLIC_NAMES[i].equals(bundles[j].getSymbolicName())) {
                    bundle = bundles[j];

                    // TODO Should use standard LogService
                    try {
                         logger.debug("Stopping Bundle", bundle);
                        bundle.stop();
                    } catch (Exception e) {
                         logger.error("Stopping Bundle", bundle, e);
                    }
                }
            }
        }
    }


    static class SimpleLogListenerImpl implements LogListener {

        /*
         * (non-Javadoc)
         *
         * @see org.osgi.service.log.LogListener#logged(org.osgi.service.log.LogEntry)
         */
        public void logged(final LogEntry entry) {
            String logLevel = null;
            switch (entry.getLevel()) {
            case LogService.LOG_DEBUG:
                logLevel = "DEBUG";
                break;

            case LogService.LOG_INFO:
                logLevel = "INFO";
                break;
            case LogService.LOG_WARNING:
                logLevel = "WARNING";
                break;
            case LogService.LOG_ERROR:
                logLevel = "ERROR";
                break;
            default:
                logLevel = "INFO";
            }

            DateFormat dateFormat = DateFormat.getTimeInstance();
            System.out.println(logLevel
                    + " - "
                    + dateFormat.format(new Date(entry.getTime()))
                    + " ["
                    + (entry.getServiceReference() != null ? "S: "
                            + entry.getServiceReference().getProperty(Constants.SERVICE_PID) : "B: "
                            + entry.getBundle().getSymbolicName()) + "] - " + entry.getMessage() + ".");
            if (entry.getException() != null) {
                System.out.println(" Exception: " + entry.getException().getMessage());
                entry.getException().printStackTrace(System.out);
            }
        }
    }
}
